//****************************************************************************************
//* 	rmld_node.c
//*	version 1.0
//*	date	jan 21, 2012 (no changes made on july, 2013)
//*	Description: ROS node for the RMLD
//****************************************************************************************


#include <asq_node/aqs_node.h>
#include <aqs_node/THD.h>

//-----------------------------------------------------------------------------------------
//	constructor
//	public
//-----------------------------------------------------------------------------------------
chemNode::chemNode(const char *port_name){
	this->sPort=port_name;
	this->connect();	

}

//-----------------------------------------------------------------------------------------
//	advertise: Advertises the topics published by this 
//		node through the ros node handler "n"
//	public
//-----------------------------------------------------------------------------------------
void chemNode::advertise(ros::NodeHandle n){
	this->thd = n.advertise<aqs_node::THD>("thd", 1000);
	this->signal = n.advertise<std_msgs::Float32>("net_strength", 1000);
	#if HAS_SERVICES
	this->service1 = n.advertiseService("enose_command", &chemNode::processServices,this);
	#endif
}

//-----------------------------------------------------------------------------------------
//	publishTopics: This function publish the topics 
//		associated with this node	
//	public
//-----------------------------------------------------------------------------------------
void chemNode::publishTopics(void){

	std::string		rxPack;
	//ARAIN: change here if you want to setup different scanning.
	if (this->isConnected){
		try{ this->serial_port->readLine(&rxPack, RUNNING_TIMEOUT); }
		catch(cereal::Exception& e){
			ROS_ERROR("Timeout - AQS_NODE");
			}
		this->serial_port->flush();

		int rx_size=rxPack.size();
		

		if (rx_size >= PACK_SIZE){

			int temp_end, hum_end,dust_end;
			temp_end=rxPack.find(";");
			hum_end=rxPack.find(";",temp_end+1);
			//dust_end=rxPack.find(";",hum_end+1);
			float temp_f=atof(rxPack.substr(0,temp_end).c_str());
			float hum_f=atof(rxPack.substr(temp_end+1,hum_end-temp_end-1).c_str());
			//float dust_f=atof(rxPack.substr(hum_end+1,dust_end-hum_end-1).c_str());
			//float signal_f=atof(rxPack.substr(dust_end+1,rxPack.size()-dust_end).c_str());
			float dust_f=atof(rxPack.substr(hum_end+1,rxPack.size()-hum_end).c_str());
			
			std_msgs::Float32 hum_msg,temp_msg,dust_msg,signal_msg;
      aqs_node::THD thd_msg;
      thd_msg.temperature = temp_f;
      thd_msg.humidity = hum_f;
      thd_msg.dust = dust_f;
      thd_msg.header.stamp = ros::Time::now();
      thd_msg.header.frame_id = "usb_sensor_frame";
			
      thd.publish(thd_msg);
			
		}
	}
	else{
		ROS_FATAL("%s not found during initialization",SENSOR_NAME);
	}


}




//-----------------------------------------------------------------------------------------
//	processServices: If enabled, this function handles the 
//		service requests
//	public
//-----------------------------------------------------------------------------------------
void chemNode::disconnect(void){

	
	if (this->isConnected){
		this->serial_port->close();
	}
	this->isConnected=false;

}


//-----------------------------------------------------------------------------------------
//	~chemNode: Class destructor
//	public
//-----------------------------------------------------------------------------------------

chemNode::~chemNode()	{

	if (this->isConnected){
		this->serial_port->close();
	}
	this->isConnected=false;
}


//---------------------------------------------------------------
//	connect: stablish a link with a device connected at the 
//		indicated serial port
//	private
//---------------------------------------------------------------

void chemNode::connect(void){
	this->isConnected=0;
	this->serial_port = new cereal::CerealPort();



	this->baudRate=DEFAULT_BAUD_RATE;
	
	std::string rxPack;	

	std::string port_name_string;
	port_name_string=this->sPort;
	bool connect_flag=true;
	try{ this->serial_port->open(port_name_string.c_str(), this->baudRate); }
		catch(cereal::Exception& e){
			ROS_FATAL("Could not connect to the device in %s", port_name_string.c_str());
			//ROS_BREAK();
			connect_flag=false;
		}
		
	if(connect_flag) {
		this->isConnected=true;
		ROS_INFO("-- Succesfully connected to device in %s",this->sPort);
		
		try{ this->serial_port->readLine(&rxPack, RUNNING_TIMEOUT); }
		catch(cereal::Exception& e){

			}

		
	}
	else	{
		this->isConnected=false;
		ROS_FATAL("Device %s Not found",SENSOR_NAME);
	}
	

}





