#include "thd_mapping/basic_node.h"


namespace thd_mapping {
    BasicNode::BasicNode() { }
    BasicNode::BasicNode (std::string pName) : BasicNode(pName, Position(0,0),
                                                         NULL)
    { /*noop*/ }

    BasicNode::BasicNode(std::string pName,
                         boost::function<void(const SensorNode&)> pCallback )  : BasicNode(pName, Position(0,0), pCallback)
    { /* noop */ }

    BasicNode::BasicNode(std::string pName, Position pPos) : BasicNode(pName, pPos, NULL)
    { /* noop */ }

    BasicNode::BasicNode(std::string pName,
                         Position pPos,
                         boost::function<void(const SensorNode&)> pCallback) : mName(pName), mPosition(pPos), mCallback(pCallback), _hasPosition(false)
    {
        if (ros::isInitialized())            
        {
            ros::NodeHandle nh(mName);
            this->mSubThd = nh.subscribe("thd", 10, &BasicNode::thdCallback, this);
            this->mSubPos = nh.subscribe("pos", 10, &BasicNode::posCallback, this);
        }
    }
    
    void BasicNode::thdCallback(const aqs_node::THD::ConstPtr& msg)
    {
        this->mReading = Reading(msg->temperature, msg->humidity, msg->dust);
        if (mCallback && _hasPosition) mCallback(*this);
    }

    void BasicNode::posCallback(const geometry_msgs::PoseStamped::ConstPtr& msg)
    {
        _hasPosition = true;
        this->mPosition = Position(msg->pose.position.x, msg->pose.position.y);
    }
    Position BasicNode::getPosition() const { return mPosition; }
    Reading BasicNode::getReading() const { return mReading; }
}
