#include <thd_mapping/kernel.h>

namespace thd_mapping {


std::vector<float> createGaussianKernel(int kernelSize, float sigma) {

  assert(kernelSize % 2 == 1); //kernelSize is odd
  std::vector<float> kernel(kernelSize*kernelSize);

  float s = 2.0 * sigma * sigma;
  float sum = 0.0;
  int halfKernelSize = (kernelSize / 2);

  glm::ivec2 xy;
  for (xy.y = -halfKernelSize; xy.y <= halfKernelSize; xy.y++)
  {
    for (xy.x = -halfKernelSize; xy.x <= halfKernelSize; xy.x++)
    {
      float r = glm::length(glm::vec2(xy));
      float& kernelValue = kernel[(xy.x + halfKernelSize) + (xy.y + halfKernelSize) * kernelSize];
      kernelValue = (exp(-(r*r) / s)) / (glm::pi<float>() * s);
      sum += kernelValue;
    }
  }
  for (int i = 0; i < kernelSize; ++i)
    for (int j = 0; j < kernelSize; ++j)
        kernel[j + i * kernelSize] /= sum;

  return kernel;

}

}
