#include "thd_mapping/median_node.hpp"
#include <algorithm>
#include <numeric>


namespace thd_mapping {
    template<typename T>
    double getMedian(std::vector<T> it)
    {
        std::nth_element(it.begin(), it.begin() + it.size()/2, it.end());
        return it[it.size()/2];
    }

    template<typename T>
    double getStddev(std::vector<T> it)
    {
        double sum = std::accumulate(it.begin(), it.end(), 0.0);
        double mean = sum / it.size();

        double sq_sum = std::inner_product(it.begin(), it.end(), it.begin(), 0.0);
        double dev = std::sqrt(sq_sum / it.size() - mean * mean);
        return dev;
    }
    MedianNode::MedianNode (std::string pName, int pWindow) : MedianNode(pName, Position(0,0), NULL, pWindow)
    { /*noop*/ }

    MedianNode::MedianNode(std::string pName,
                           boost::function<void(const SensorNode&)> pCallback, int pWindow )  : MedianNode(pName, Position(0,0), pCallback, pWindow)
    { /* noop */ }

    MedianNode::MedianNode(std::string pName, Position pPos, int pWindow) : MedianNode(pName, pPos, NULL, pWindow)
    { /* noop */ }

    MedianNode::MedianNode(std::string pName,
                                 
                           Position pPos,
                           boost::function<void(const SensorNode&)> pCallback, int pWindow) : BasicNode(pName, pPos, pCallback), mWindow(pWindow)
    {
        // if (ros::isInitialized())
        // {
        //     ros::NodeHandle nh(pName);
        //     this->mSubThd = nh.subscribe("thd", 10, &MedianNode::thdCallback, this);
        //     this->mSubPos = nh.subscribe("pos", 10, &MedianNode::posCallback, dynamic_cast<BasicNode*>(this));
        // }
    }

    void MedianNode::thdCallback(const aqs_node::THD::ConstPtr& msg)
    {
        double temp = msg->temperature, humidity = msg->humidity, dust = msg->dust;

        double median = thd_mapping::getMedian(this->mListTemperatures);
        double dev = thd_mapping::getStddev(this->mListTemperatures);
        double diff = abs(temp - median);
        if (diff > (3.0 * dev))
        {
            temp = median;
        }

        median = thd_mapping::getMedian(this->mListHumidity);
        dev = thd_mapping::getStddev(this->mListHumidity);
        diff = abs(humidity - median);
        if (diff > (3.0 * dev))
        {
            humidity = median;
        }

        
        median = thd_mapping::getMedian(this->mListDust);
        dev = thd_mapping::getStddev(this->mListDust);
        diff = abs(dust - median);
        if (diff > (3.0 * dev))
        {
            dust = median;
        }

        
        this->mListTemperatures.push_back(temp);
        this->mListHumidity.push_back(humidity);
        this->mListDust.push_back(dust);
        
        if (mListTemperatures.size() > mWindow) mListTemperatures.erase(mListTemperatures.begin());
        if (mListHumidity.size() > mWindow) mListHumidity.erase(mListHumidity.begin());
        if (mListDust.size() > mWindow) mListDust.erase(mListDust.begin());
        
        this->mReading = Reading(temp,
                                 humidity,
                                 dust);
        if (mCallback && _hasPosition)
            mCallback(*this);
    }
}
