
#include <string>
#include <thd_mapping/sensor_node.h>
#include <thd_mapping/basic_node.h>
#include <thd_mapping/smoothing_node.hpp>
#include <thd_mapping/median_node.hpp>
#include <glm/vec3.hpp>
#include <memory>

#include "thd_mapping/util.h"
#include <thd_mapping/gaussian_kernel_weight_mapper.h>

int main(int argc, char *argv[])
{
  glm::ivec2 dim = { 200,200 };
  float initTemp = 22.0f;
  float initHumidity = 30.0f;
  float initDust = 20.0f;

  auto mappingAlgorithm = std::shared_ptr<thd_mapping::GaussianKernelWeightMapper<3>>(
    new thd_mapping::GaussianKernelWeightMapper<3>(
      std::array <thd_mapping::GridMap, 3> ({
        thd_mapping::GridMap(std::vector<float>(dim.x*dim.y, initTemp), dim, { 0,0,0 }, 0.1f),
        thd_mapping::GridMap(std::vector<float>(dim.x*dim.y, initHumidity), dim, { 0,0,0 }, 0.1f),
        thd_mapping::GridMap(std::vector<float>(dim.x*dim.y, initDust), dim, { 0,0,0 }, 0.1f)
      }),
    15.0f)
  );

  std::vector<thd_mapping::DataPoint<3>> dataPoints = thd_mapping::dataPointsFromCSV2("/home/ace/workspace/RIS_IPW_2016/tools/scripts/output.csv");
  // dataPoints = std::vector<thd_mapping::THDataPoint>(dataPoints.end()-100*3,dataPoints.end());

  int step = 100 * 3;
  for(int i = 0; i < dataPoints.size() - step; i += 100*3)
  {
    mappingAlgorithm->addDataPoints(std::vector<thd_mapping::DataPoint<3>>(dataPoints.begin()+i,dataPoints.begin() + i + step));
    std::stringstream ss, ss2;
    ss << "img/img-temp-" << (dataPoints.begin()+i)->time << ".ppm";
    thd_mapping::writePPMFromGridMap(mappingAlgorithm->getMaps()[0],ss.str());
    ss2 << "img/img-hum-" << (dataPoints.begin()+i)->time << ".ppm";
    thd_mapping::writePPMFromGridMap(mappingAlgorithm->getMaps()[1],ss.str());
    sleep(1);
  }

  thd_mapping::writePPMFromGridMap(mappingAlgorithm->getMaps()[0],"img/imgGTemp.ppm");
  thd_mapping::writePPMFromGridMap(mappingAlgorithm->getMaps()[1],"img/imgGHum.ppm");
  // thd_mapping::writePPMFromGridMap(mappingAlgorithm2->getMap(),"img2.ppm");

  return 0;

}

