#include <ros/ros.h>
#include <ros/time.h>
#include <nav_msgs/OccupancyGrid.h>
#include <string>

#include <string>
#include <thd_mapping/sensor_node.h>
#include <thd_mapping/basic_node.h>
#include <thd_mapping/smoothing_node.hpp>
#include <thd_mapping/median_node.hpp>
#include <glm/vec3.hpp>
#include <memory>

#include "thd_mapping/util.h"
#include <thd_mapping/gaussian_kernel_weight_mapper.h>

#define TEST_MAPPING_IMAGE
ros::Publisher measurementMapPub;
ros::Subscriber mapSub;

thd_mapping::GridMap gMap;
bool ghasGridMap = false;

void publishMap(const thd_mapping::GridMap& gridMap, float minValue, float maxValue);


void mapCallback(const nav_msgs::OccupancyGrid::ConstPtr& msg)
{
  ROS_INFO("Received Map.");
  glm::ivec2 lDim = {
    lDim.x = msg->info.width,
    lDim.y = msg->info.height
  };

  gMap = thd_mapping::GridMap(
        std::vector<float>(&msg->data[0], &msg->data[0] + msg->data.size()),
      // std::move(grid),
      lDim,
      glm::vec2(msg->info.origin.position.x, msg->info.origin.position.y),
      msg->info.resolution);
  ghasGridMap = true;
}

int main(int argc, char *argv[])
{
  ros::init(argc, argv, "test_gkwm_node");
  ros::NodeHandle n;
  ros::Rate r(1);
  measurementMapPub = n.advertise<thd_mapping::MeasurementMap>("/measurement_map", 1000);
  mapSub = n.subscribe("/map", 1, &mapCallback);

  while (n.ok() && !ghasGridMap)
  {
    ROS_INFO("Waiting for /map");
    ros::spinOnce();
    r.sleep();
  }

  glm::ivec2 dim = { 200,200 };
  float initTemp = 15.0f;
  float initHumidity = 30.0f;
  float initDust = 20.0f;

  std::vector <thd_mapping::GDMAlgorithmPtr> algorithms;
  algorithms.push_back(thd_mapping::GDMAlgorithmPtr(new thd_mapping::GaussianKernelWeightMapper(thd_mapping::GridMap(std::vector<float>(dim.x*dim.y, initTemp), dim, { -5,-5 }, 0.1f), 31.0f)));
  algorithms.push_back(thd_mapping::GDMAlgorithmPtr(new thd_mapping::GaussianKernelWeightMapper(thd_mapping::GridMap(std::vector<float>(dim.x*dim.y, initHumidity), dim, { -5,-5 }, 0.1f), 31.0f)));
  algorithms.push_back(thd_mapping::GDMAlgorithmPtr(new thd_mapping::GaussianKernelWeightMapper(thd_mapping::GridMap(std::vector<float>(dim.x*dim.y, initDust), dim, { -5,-5 }, 0.1f), 31.0f)));

//   boost::function<void(const thd_mapping::SensorNode&)> func = boost::bind(&callback, _1, algorithms);

  boost::function<void(const thd_mapping::SensorNode&)> func5 = boost::bind(&thd_mapping::dataPointCallback, _1, algorithms, 1);
  boost::function<void(const thd_mapping::SensorNode&)> func9 = boost::bind(&thd_mapping::dataPointCallback, _1, algorithms, 5);
  boost::function<void(const thd_mapping::SensorNode&)> funcusb = boost::bind(&thd_mapping::dataPointCallback, _1, algorithms, 9);



  std::vector<std::shared_ptr<thd_mapping::SensorNode>> nodes
  = {
    std::make_shared<thd_mapping::BasicNode>(std::string("n5"), thd_mapping::Position(0,0), func5),
    std::make_shared<thd_mapping::BasicNode>(std::string("n9"), thd_mapping::Position(0,5), func9),
    std::make_shared<thd_mapping::BasicNode>(std::string("usb"), thd_mapping::Position(0,5), funcusb)
  };


  while (n.ok())
  {
    ROS_INFO("In ros loop");
    ros::spinOnce();
    // publishMap(algorithms[0]->getMap(),0.0,50.0);
    measurementMapPub.publish(algorithms[0]->getMap().toMeasurementMap());
    r.sleep();
  }
}

