ScaleView = function (options) {
    options = options || {};

    that = this;
    this.alpha = options.alpha || 1.0;
    this.hidden = options.hidden || false;
    this.maxValue = 100;
    this.minValue = 0;

    this.draw = function (canvas,context) {
        if (this.hidden)
            return
        
        let sizeX = 32;
        let x = canvas.width-sizeX-3;
        let y = canvas.height*0.25;        
        let sizeY = canvas.height*0.5;
        
        var grad = context.createLinearGradient(x,y,x,y+sizeY);
        grad.addColorStop(0,"white");
        grad.addColorStop(1,"black");

        context.fillStyle=grad;
        context.fillRect(x,y,sizeX,sizeY);

        let max = this.maxValue.toFixed(2);
        let min = this.minValue.toFixed(2);

        context.fillStyle="black";
        context.font="14px Georgia";
        context.textAlign="center"; 
        context.fillText(max,x+sizeX/2,y+15);
        context.fillStyle="white";
        context.fillText(min,x+sizeX/2,y+sizeY-5);
        
        context.strokeStyle="black";
        context.rect(x-1,y-1,sizeX+2,sizeY+2);
        context.stroke();
    }
};

MeasurementMapView = function (options) {
    options = options || {};

    this.alpha = options.alpha || 1.0;
    this.canvas2 = document.createElement("canvas");
    this.oldwidth = 0;
    this.oldheight = 0;
    this.hidden = options.hidden || false;

    this.draw = function (context, offset, scale, resolution, canvas) {
        if (!this.message || this.hidden)
            return

        this.width = this.message.info.width;
        this.height = this.message.info.height;
        this.scaleX = this.message.info.resolution;
        this.scaleY = this.message.info.resolution;

        if(this.width != this.oldwidth || this.height != this.oldheight)
        {
            this.imageData = context.createImageData(this.width, this.height);
            this.oldwidth = this.width;
            this.oldheight = this.height;
        }



        // var canvas2 = document.createElement("canvas");
        this.canvas2.width = this.width;
        this.canvas2.height = this.height;
        var ctx2 = this.canvas2.getContext("2d");


        for (var row = 0; row < this.height; row++) {
            for (var col = 0; col < this.width; col++) {
                // determine the index into the map data
                var mapI = col + ((this.height - row - 1) * this.width);

                // determine the value
                var data = this.message.data[mapI];
                var val = data*2.55;

                // determine the index into the image data array
                var i = (col + (row * this.width)) * 4;
                // r
                this.imageData.data[i] = val;
                // g
                this.imageData.data[++i] = val;
                // b
                this.imageData.data[++i] = val;
                // a
                this.imageData.data[++i] = 255 ;
            }
        }
        ctx2.putImageData(this.imageData, 0, 0);

        let resolutionFactor = this.scaleX / resolution;

        this.x = this.message.info.origin.position.x ; 
        this.y = -this.message.info.origin.position.y  - this.height * this.message.info.resolution;

        context.drawImage(this.canvas2,
            offset.x + this.x / this.scaleX * scale * resolutionFactor,
            offset.y + this.y / this.scaleY * scale * resolutionFactor,
            this.canvas2.width * scale * resolutionFactor ,
            this.canvas2.height * scale * resolutionFactor 
        );
    }
};