import rosbag
import rospy
import csv
import os
from aqs_node.msg import THD

def thd_to_str(thd):
  assert(isinstance(thd, THD))
  return "Temperature: " + str(thd.temperature) + ", Humidity: " + str(thd.humidity) + "@ " + str(thd.header.stamp)


class NodeData:
  def __init__(self, name, thd):
    self.name = name
    self.thd = thd

  def __repr__(self):
     return self.name + ":" + thd_to_str(self.thd)

class DatabaseToROSBag:
  def __init__(self, out_file_name = "output.bag"):
    self.file_name = out_file_name
    self.bag = rosbag.Bag(self.file_name, 'w')
    self.node5_data = []
    self.node9_data = []
    self.usb_data = []

  def read_csv(self, in_file_name):
    pass_1 = True
    with open(in_file_name, 'rb') as csvfile:
      csv_reader = csv.reader(csvfile, delimiter=' ', quotechar='|')
      for row in csv_reader:
        if pass_1:
          pass_1 = False
          continue
        row_ = (', '.join(row).split(","))
        thd_data = THD()
        thd_data.header.stamp = rospy.Time.from_sec(int(row_[0]))
        thd_data.temperature = float(row_[2])
        thd_data.humidity = float(row_[3])
        assert(row_[1] == "N5")
        self.node5_data.append(NodeData(row_[1].lower(), thd_data))

        thd_data = THD()
        thd_data.header.stamp = rospy.Time.from_sec(int(row_[0]))
        thd_data.temperature = float(row_[5])
        thd_data.humidity = float(row_[6])
        assert (row_[4] == "N9")
        self.node9_data.append(NodeData(row_[4].lower(), thd_data))

        thd_data = THD()
        thd_data.header.stamp = rospy.Time.from_sec(int(row_[0]))
        thd_data.temperature = float(row_[8])
        thd_data.humidity = float(row_[9])
        assert (row_[7] == "USB")
        self.usb_data.append(NodeData(row_[7].lower(), thd_data))


  def write_data_to_bag(self):
    for datapoint_n5, datapoint_n9, datapoint_usb in zip(self.node5_data, self.node9_data, self.usb_data):
      assert(isinstance(datapoint_n5, NodeData))
      self.bag.write(datapoint_n5.name + "/thd", datapoint_n5.thd)
      assert(isinstance(datapoint_n9, NodeData))
      self.bag.write(datapoint_n9.name + "/thd", datapoint_n9.thd)
      assert(isinstance(datapoint_usb, NodeData))
      self.bag.write(datapoint_usb.name + "/thd", datapoint_usb.thd)

    self.bag.close()
    self.bag.reindex()


  def print_usb_data(self):
    print self.usb_data

  def print_node5_data(self):
    print self.node5_data

  def print_node9_data(self):
    print self.node9_data


if __name__ == "__main__":
  db_to_rosbag = DatabaseToROSBag('data.bag')
  db_to_rosbag.read_csv("output.csv")
  db_to_rosbag.write_data_to_bag()